/*************************************************************************************
 Data Visualization - Assignment 2                                           
 
 Author: Ming-Yu Lin
 Purpose: Show popularity for Harry Potter Series from 2008 to 2015     
 
 Usage: 1. A mouse click on month will show the bar plot for specific month.
 
 Program: Processing 3.3.6
 *************************************************************************************/

Table table;
int rows, cols;
float radius;
int dataMatrix[][];
float maxValues[];
float minValues[];
String[] titles;
int year;
boolean[] dateChecked;
String[] dates;
String[] years;
color[] colors;
int constHorizontalMargin = 25;
int constVerticalMargin = 70;
float[] coordinate_x;
float coordinate_y;
float cellWidth;
float cellHeight;
PImage img;


void setup() {
  //size(1080, 600);
  fullScreen();
  table = loadTable("data/output2.csv");
  img = loadImage("data/Light-Grey-Background-Hd-5-1.jpg");
  rows = table.getRowCount();
  cols = table.getColumnCount();
  println(rows);
  println(cols);
  dataMatrix = new int[rows-1][cols-2];
  titles = new String[rows-1];
  dateChecked = new boolean[cols-2];
  maxValues = new float[rows-1];
  minValues = new float[rows-1];
  dates = new String[cols-2];
  years = new String[(cols-2)/12];
  colors = new color[cols-2];
  colors[0] = #FF0000;
  colors[1] = #FF7F00;
  colors[2] = #FFFF00;
  colors[3] = #00FF00;
  colors[4] = #0000FF;
  colors[5] = #4B0082;
  colors[6] = #9400D3;
  // Load data to dataMatrix
  for (int i=0; i < rows -1; ++i){
    for(int j=0; j < cols - 2; ++j){
      dataMatrix[i][j] = table.getInt(i+1, j+2);
      if (dataMatrix[i][j] > maxValues[i]){
        maxValues[i] = dataMatrix[i][j];
       }
       if (i == 0) {
         dates[j] = table.getString(i,j+2);
         dateChecked[j] = false;
       }
       if ( j % 12 == 0){
         years[j/12] = dateParser(dates[j])[0];
       }
    }
    titles[i] = table.getString(i+1,0).substring(21);
    titles[i] = titles[i].substring(0,1).toUpperCase() + titles[i].substring(1);
  }
  for(int i=0; i<dataMatrix.length; i++){
    minValues[i] = maxValues[i];
    for(int j=0; j<dataMatrix[0].length; j++){
      if(dataMatrix[i][j] < minValues[i]){
        minValues[i] = dataMatrix[i][j];
      }
    }
  }
  noLoop();
}

void draw() {
  background(239, 243, 249);
  int textOffset = 40;
  coordinate_x = new float[cols-2];
  cellWidth = (width - 2 * constHorizontalMargin)/(cols - 2);
  cellHeight = (height - constVerticalMargin*2 - textOffset * 2) / (rows - 1);
  

  // year at the bottom
  fill(#4747d1);
  for(int i = 0; i < years.length; i++){
    textSize(30);
    textAlign(LEFT);
    text(years[i], cellWidth*(i*12 + 6), height - constVerticalMargin);
  }
  coordinate_y = height - textOffset*2 - constVerticalMargin;
  print(coordinate_y);
  // print month
  fill(#4747d1);
  for(int i = 0; i < dates.length; i++){
    textSize(15);
    textAlign(LEFT);
    text(dateParser(dates[i])[1], cellWidth*i + 2*constHorizontalMargin, height - textOffset*2 - constVerticalMargin);
    coordinate_x[i] = cellWidth*i + 2*constHorizontalMargin;
  }  
  // Harry Potter
  textSize(40);
  fill(#4747d1);
  text("Harry Potter Books",constHorizontalMargin,40);
  // for title text
  fill(#4747d1);
  for(int i =0; i < rows-1; i++) {
    textSize(30);
    textAlign(LEFT);
    text(titles[i], 32, cellHeight*i+constVerticalMargin+10);
  }
  
  // Line for base
  strokeWeight(10);
  stroke(107);
  line(constHorizontalMargin, height- textOffset * 2 - 32 - constVerticalMargin, width - constHorizontalMargin, height - textOffset * 2 - 32 - constVerticalMargin);
  if(any(dateChecked) == -1){
    // draw every year
    for(int i=0; i<dataMatrix.length; i++){
      for(int j=0; j<dataMatrix[0].length; j++){
        float greyScale = map(dataMatrix[i][j], minValues[i], maxValues[i], 100, 240);
        float rectWidth = map(dataMatrix[i][j], minValues[i], maxValues[i], 5, 50);
        strokeWeight(1);
        stroke(colors[i]);
        fill(colors[i],greyScale);
        rectMode(CENTER);
        rect(cellWidth*j + 2*constHorizontalMargin,
             cellHeight*i+ constVerticalMargin + height / 16,
             rectWidth,rectWidth);
      }
    }
  } else {
    // draw bar plot for each year
    int j = any(dateChecked);
    textSize(40);
    fill(#4747d1);
    text(dates[j],coordinate_x[(cols-2)/2],50);
    for(int i=0; i<dataMatrix.length; i++){
      float greyScale = map(dataMatrix[i][j], minValues[i], maxValues[i], 100, 240);
      float rectWidth = 50.0;
      float rectLength = map(dataMatrix[i][j], minValues[i], maxValues[i], 5, width - 2*constHorizontalMargin-150);
      strokeWeight(1);
      stroke(colors[i]);
      fill(colors[i],greyScale);
      rectMode(CORNER);
      rect(coordinate_x[0] - 10 ,
           cellHeight*i+ constVerticalMargin + height / 25,
           rectLength,rectWidth);
      fill(#4747d1);
      textSize(28);
      textAlign(RIGHT);
      text(dataMatrix[i][j],rectLength + 125,cellHeight*i+ constVerticalMargin + height / 14);
    }
  }
}

void mousePressed(){
  println(mouseY);
  println(coordinate_y);
  boolean onMonth = false;
  for(int i = 0; i < cols-2; i++){
    if (mouseX > coordinate_x[i] - cellWidth/2 && mouseX < coordinate_x[i] + cellWidth/2 &&
        mouseY < coordinate_y +20 && mouseY > coordinate_y -20){
      datesChecked(dateChecked,i);
      onMonth = true;
      break;
    }
  }
  if (!onMonth){
    datesChecked(dateChecked,cols-1);
  }
  redraw();
}